/*
 * Write to display memory.
 *
 * Copyright (C) 2017 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdint.h>
#include "mips.h"
#include "vga.h"

void init_framebuffer(uint32_t *data)
{
	uint32_t *addr = (uint32_t *) SCREEN_BASE_KSEG0;
	uint16_t x, y;

	for (y = 0; y < LINE_COUNT; y++)
	{
		for (x = 0; x < LINE_LENGTH; x += 4)
		{
			*addr = *data;
			addr++;
			data++;
		}
	}
}

void init_framebuffer_with_pattern()
{
	uint32_t *addr = (uint32_t *) SCREEN_BASE_KSEG0;
	uint32_t base, value;
	uint16_t x, y;
	uint8_t row, offset;

	for (y = 0; y < LINE_COUNT; y++)
	{
		/* Start each line with a base value. */

		row = y / 16;
		base = (row << 26) | (row << 18) | (row << 10) | (row << 2);

		/* Set the intensity bit on every other palette row. */

		if (row % 2) base |= 0x80808080;

		for (x = 0; x < LINE_LENGTH; x += 4)
		{
			/* Combine the base value with the offset. */

			offset = x / 20;
			value = base | (offset << 24) | (offset << 16) | (offset << 8) | offset;

			/* Store the value. */

			*addr = value;

			/* Access the next word. */

			addr++;
		}
	}
}
